# -*- coding: utf-8 -*-
# Copyright 2009-2014, Red Hat, Inc.
# License: GPL-2.0+ <http://spdx.org/licenses/GPL-2.0+>
# See the LICENSE file for more details on Licensing

import pytest
import os
import copy
from dingus import Dingus

from libtaskotron import main
from libtaskotron import config


@pytest.mark.usefixtures('setup')
class TestProcessArgs():

    @pytest.fixture
    def setup(self, monkeypatch):
        self.ref_uuid = '20150930_153933_359680'
        self.ref_artifactsdir = '/dir/'
        self.stub_config = Dingus()
        self.stub_get_config = Dingus()

        self.stub_get_config.return_value = self.stub_config
        monkeypatch.setattr(config, 'get_config', self.stub_get_config)
        self.stub_config.artifactsdir = self.ref_artifactsdir

    def test_process_args_koji_build(self):
        ref_input = {'arch': ['x86_64'],
                     'item': 'foo-1.2-3.fc99',
                     'type': 'koji_build',
                     'task': 'sometask.yml',
                     'ssh': None,
                     'uuid': '',
                     'override': []}

        ref_args = copy.deepcopy(ref_input)
        ref_args['koji_build'] = 'foo-1.2-3.fc99'
        ref_args['artifactsdir'] = self.ref_artifactsdir+self.ref_uuid

        test_args = main.process_args(ref_input)
        ref_args['artifactsdir'] = os.path.join(config.get_config().artifactsdir, ref_args['uuid'])
        assert test_args == ref_args

    def test_process_args_bodhi_id(self):
        ref_input = {'arch': ['x86_64'],
                     'item': 'foo-1.2-3.fc99',
                     'type': 'bodhi_id',
                     'task': 'sometask.yml',
                     'ssh': None,
                     'uuid': '',
                     'override': []}

        ref_args = copy.deepcopy(ref_input)
        ref_args['bodhi_id'] = 'foo-1.2-3.fc99'
        ref_args['artifactsdir'] = self.ref_artifactsdir+self.ref_uuid

        test_args = main.process_args(ref_input)
        ref_args['artifactsdir'] = os.path.join(config.get_config().artifactsdir, ref_args['uuid'])
        assert test_args == ref_args

    def test_process_args_koji_tag(self):
        ref_input = {'arch': ['x86_64'],
                     'item': 'dist-fc99-updates',
                     'type': 'koji_tag',
                     'task': 'sometask.yml',
                     'ssh': None,
                     'uuid': '',
                     'override': []}

        ref_args = copy.deepcopy(ref_input)
        ref_args['koji_tag'] = 'dist-fc99-updates'
        ref_args['artifactsdir'] = self.ref_artifactsdir+self.ref_uuid

        test_args = main.process_args(ref_input)
        ref_args['artifactsdir'] = os.path.join(config.get_config().artifactsdir, ref_args['uuid'])
        assert test_args == ref_args

    def test_process_args_compose(self):
        ref_input = {'arch': [''],
                     'item': 'rawhide',
                     'type': 'compose',
                     'task': 'sometask.yml',
                     'ssh': None,
                     'uuid': '',
                     'override': []}

        ref_args = copy.deepcopy(ref_input)
        ref_args['compose'] = 'rawhide'
        ref_args['artifactsdir'] = self.ref_artifactsdir+self.ref_uuid

        test_args = main.process_args(ref_input)
        ref_args['artifactsdir'] = os.path.join(config.get_config().artifactsdir, ref_args['uuid'])
        assert test_args == ref_args

    def test_process_args_koji_tag_multiple_arches(self):
        ref_input = {'arch': ['x86_64', 'i386', 'noarch'],
                     'item': 'dist-fc99-updates',
                     'type': 'koji_tag',
                     'task': 'sometask.yml',
                     'ssh': None,
                     'uuid': '',
                     'override': []}

        ref_args = copy.deepcopy(ref_input)
        ref_args['koji_tag'] = 'dist-fc99-updates'
        ref_args['artifactsdir'] = self.ref_artifactsdir+self.ref_uuid

        test_args = main.process_args(ref_input)
        ref_args['artifactsdir'] = os.path.join(config.get_config().artifactsdir, ref_args['uuid'])
        assert test_args == ref_args

    def test_process_args_no_arch(self):
        ref_input = {'type': 'invalid',
                     'arch': None,
                     'ssh': None,
                     'uuid': '',
                     'override': []}

        ref_args = copy.deepcopy(ref_input)
        ref_args['arch'] = ['noarch']
        ref_args['artifactsdir'] = self.ref_artifactsdir+self.ref_uuid

        test_args = main.process_args(ref_input)
        ref_args['artifactsdir'] = os.path.join(config.get_config().artifactsdir, ref_args['uuid'])
        assert test_args == ref_args


@pytest.mark.usefixtures('setup')
class TestCheckArgs():

    @pytest.fixture
    def setup(self):
        self.empty_args = {'local': None, 'libvirt': None, 'ssh': None}

    def test_runmode_exclusive(self):
        '''allow only single runmode selection'''
        ref_inputdata_bad = [{'local': True, 'ssh': 'user@machine'},
                             {'local': True, 'libvirt': True},
                             {'libvirt': True, 'ssh': 'user@machine'},
                             {'local': True, 'ssh': 'user@machine', 'libvirt': True}]
        ref_inputdata_good = [{'local': True},
                              {'ssh': 'user@machine'},
                              {'libvirt': True},
                              {}]

        for ref_inputdata in ref_inputdata_bad:
            stub_error = Dingus()
            stub_parser = Dingus()
            stub_parser.error = stub_error
            args = self.empty_args.copy()
            args.update(ref_inputdata)
            main.check_args(stub_parser, args)
            assert len(stub_error.calls()) == 1

        for ref_inputdata in ref_inputdata_good:
            stub_error = Dingus()
            stub_parser = Dingus()
            stub_parser.error = stub_error
            args = self.empty_args.copy()
            args.update(ref_inputdata)
            main.check_args(stub_parser, args)
            assert len(stub_error.calls()) == 0

    def test_ssh_bad_usermachine(self):
        '''fail on invalid user@machine[:port] specifier'''
        ref_inputdata = {'ssh': 'user#machine'}

        stub_error = Dingus()
        stub_parser = Dingus()
        stub_parser.error = stub_error
        args = self.empty_args
        args.update(ref_inputdata)
        main.check_args(stub_parser, args)

        assert len(stub_error.calls()) == 1
